<?php

/**
 * request actions.
 *
 * @package    sf_sandbox
 * @subpackage request
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 9301 2008-05-27 01:08:46Z dwhittle $
 */
class requestActions extends sfActions
{
  public function preExecute()
  {
    if (!defined('NO_SHOW_FEE')) {
      define('NO_SHOW_FEE', Settings::getNoShowFeeStatus());
      define('NO_SHOW_FEE_COST', Settings::getNoShowFeeCost());
    }
    $this->getUser()->setCulture('en');
  }

  public function executeNew($request)
  {
    $id = $request->getParameter('relo_id');
    $listing = Doctrine::getTable('Listing')->findForAddRequest($id, $this->getUser()->getCountry());
    $this->forward404Unless($listing);

    $dates = $request->getParameter('requested_dates', array());
    sort($dates);
    $pickup = current($dates);
    $dropoff = end($dates);
    $times = $request->getParameter('request');

    return $this->redirect($listing->getRelocationQuery($this->getUser()->isAuthenticated() && $this->getUser()->getAttribute('profile')->isVerified() ? '@request_confirm' : '@request_auth', true,
                          'pickup_date=' . $pickup .
                          '&dropoff_date=' . $dropoff .
                          '&pickup_time=' . (isset($times['pickup_time']) ? $times['pickup_time'] : '0') .
                          '&dropoff_time=' . (isset($times['dropoff_time']) ? $times['dropoff_time'] : '0') . 
                          '&dropoff=' .$request->getParameter('dropoff', 0).'&'
    ));
  }

  public function executeAdd($sfRequest)
  {
    $this->forwardUnless($this->getUser()->isAuthenticated(), 'request', 'login');
    $this->forwardUnless($this->getUser()->getAttribute('profile')->isVerified(), 'main', 'activateAccount');

    $id = $sfRequest->getParameter('relo_id');
    $listing = Doctrine::getTable('Listing')->findForAddRequest($id, $this->getUser()->getCountry());
    $this->forward404Unless($listing);

    if ($req = $this->getUser()->getListingRequirements($listing)) {
      $this->required = $req;
    }

    $this->validateListing($sfRequest, $listing);

    $this->form = $this->getAddRequestForm($sfRequest, $listing);

    if ($sfRequest->isMethod('post') && $sfRequest->hasParameter('request'))
    {
      $this->form->bind($sfRequest->getParameter('request'));
    }

    if ($this->form->isValid() && ($request = $this->form->save()) !== false)
    {
      $this->dispatcher->notify(new sfEvent($request, 'request.sent'));

      $this->redirect($listing->getRelocationQuery('@request_sent').'?sent=1');
    } elseif ($this->form->isBound()) {
      TcLogger::log('request_errors', array(
        '===== START ERROR =====',
        $this->getUser()->getAttribute('profile')->get('email').' had an error with listing '.$listing->getId(),
        (string) $this->form->getErrorSchema(),
        '=====  END ERROR  ====='
      ));
    }

    $this->listing = $listing;
    $this->setMetaForListing($listing);
  }

  private function validateListing($sfRequest, $listing)
  {

    $validator = new tcListingRequestValidator(array(
      'listing' => $listing,
      'user'    => $this->getUser()
    ));

    /*
      User will be redirected if:
        - HTTP request is not POST
        - dates parameter are not sent
        - listing can't be requested (listing already requested it, or status is not pending anymore)
    */
    $this->redirectUnless(
      (
        $sfRequest->isMethod('POST') ||
        ($sfRequest->getParameter('pickup_date') && $sfRequest->getParameter('dropoff_date'))
      ) &&
      !$validator->hasError(array(
        'pickup_date' => $sfRequest->getParameter('pickup_date'),
        'dropoff_date' => $sfRequest->getParameter('dropoff_date'),
        'pickup_time' => $sfRequest->getParameter('pickup_time'),
        'dropoff_time' => $sfRequest->getParameter('dropoff_time'),
        'dropoff' => $sfRequest->getParameter('dropoff'),
      )),
      $listing->getRelocationQuery('listing/view').
        '?pickup_date='.$sfRequest->getParameter('pickup_date').
        '&dropoff_date='.$sfRequest->getParameter('dropoff_date')
    );
  }

  public function executeCost($request)
  {
    $id = $request->getParameter('relo_id');
    $listing = Doctrine::getTable('Listing')->findForAddRequest($id, $this->getUser()->getCountry());
    $this->forward404Unless($listing);

    $form = $this->getAddRequestForm($request, $listing);

    return $this->renderText(json_encode(array(
      'free_days' => $form->getRequest()->get('nb_free_days'),
      'paid_days' => $form->getRequest()->get('nb_paid_days'),
      'cost' => $listing->getRevenue($form->getRequest()),
      'type' => $listing->getVehicle()->get('rental_type'),
      'dates' => tc_format_date($form->getRequest()->get('pickup_date')).' - '.
                 tc_format_date($form->getRequest()->get('drop_off_date'))
    )));
  }

  private function getAddRequestForm($sfRequest, $listing)
  {
    return new AddRequestForm(array(
        'pickup_date'      => $sfRequest->getParameter('pickup_date'),
        'dropoff_date'     => $sfRequest->getParameter('dropoff_date'),
        'pickup_time'      => $sfRequest->getParameter('pickup_time'),
        'dropoff_time'     => $sfRequest->getParameter('dropoff_time'),
        'dropoff'          => $sfRequest->getParameter('dropoff'),
        'is_resubmit'      => $sfRequest->getParameter('resubmit', 0),
      ), array(
      'no_show_fee'        => NO_SHOW_FEE,
      'no_show_fee_amount' => NO_SHOW_FEE_COST,
      'listing'            => $listing,
      'user'               => $this->getUser()->getAttribute('profile', new User())
    ));
  }

  public function executeView($request)
  {
    $this->forwardUnless($this->getUser()->isAuthenticated(), 'request', 'login');

    $tcRequest = Doctrine::getTable('Request')->find($request->getParameter('id'));
    $this->forward404Unless($tcRequest && $tcRequest->get('driver_id') == $this->getUser()->getId());

    $this->setVar('request', $tcRequest);
    $this->listing   = $tcRequest->get('Listing');
  }

  public function executeActivateAccount($request)
  {
    $this->forward404Unless($this->getUser()->isAuthenticated());
    $this->forward404Unless($request->isMethod('POST') && $request->isXmlHttpRequest());

    $profile = $this->getUser()->getAttribute('profile');
    $profile->refresh();
    $code  = $request->getParameter('code');
    $valid = true;

    if ($profile->get('user_status') == 'inactive')
    {
      $valid = $profile->get('activation_code') == $code;
    }
    $updated = false;
    if ($valid)
    {

      // OK, code confirmed, we can log in user
      $profile->confirmEmail();
      $profile->save();
      $updated = true;
    }

    return $this->renderText(json_encode(array(
      'success' => $valid,
      'code' => $code,
      'status' => $profile->get('user_status'),
      'updated' => $updated,
      'id' => $profile->getId()
    )));
  }

  public function executeUpdateDetails($request)
  {
    $this->user_details_form = new UserDetailsForm(array(), array(
      'user' =>  $this->getUser()->getAttribute('profile'),
      'phone_verification'  => $request->getParameter('phone', false),
      'email_notifications' => $request->getParameter('email_notifications', false),
      'country'             => $this->getUser()->getUserDomain(),
    ));

    $this->forward404Unless($request->isXmlHttpRequest());

    $this->user_details_form->bind($request->getParameter('user_details'));

    if ($this->user_details_form->isValid())
    {
      $user = $this->user_details_form->save();
    }

    return $this->renderText(json_encode(array(
      'success'       => $this->user_details_form->isValid(),
      'verify_phone'  => false,
      'errors'        => $this->user_details_form->getAllErrorsArray()
    )));
  }

  public function executeLogin($request)
  {
    $listing = false;
    if (($id = $request->getParameter('relo_id', false)) != false) {
      $listing = Doctrine::getTable('Listing')->findForAddRequest($id, $this->getUser()->getCountry());
      $this->forward404Unless($listing);

      $form = $this->getAddRequestForm($request, $listing);
      $this->setVar('request', $form->getRequest());
      $this->setMetaForListing($listing);
    }

    $this->listing = $listing;
    $this->sign_in_form  = new popupSignInForm();
    $this->register_form = new RegisterForm();
  }


  public function executeCancel(sfWebRequest $request)
  {
    $tcRequest = Doctrine::getTable('Request')->find($request->getParameter('id'));
    $this->forward404Unless($tcRequest && $tcRequest->isDriverId($this->getUser()->getId()) && !$tcRequest->isStatus(Request::STATUS_CANCELED_BY_DRIVER));
    $this->forward404Unless($tcRequest->isStatus(array(Request::STATUS_PENDING, Request::STATUS_ACCEPTED)));
    $this->setVar('request', $tcRequest);

    if ($tcRequest->isAccepted())
    {
      $this->setTemplate('bookingCancel');
    }

    $this->form = new CancelRequestForm(array(), array('ask_reason' => $tcRequest->isAccepted()));

    if ($request->isMethod('POST'))
    {
      $this->form->bind($request->getParameter('request'), $request->getFiles('request'));

      if ($this->form->isValid())
      {
        $values = $this->form->getValues();
        $filename = null;
        if ($values['file']) {
          $filename = '/cancellations/'.$this->cleanFileName($tcRequest->getId().'-'.$values['file']->getOriginalName());
          $values['file']->save(sfConfig::get('sf_upload_dir').$filename);
        }
	
        $tcRequest->driverCancel(
          $this->form->getValue('comment'),
          $this->form->getValue('subcategory') ?: $this->form->getValue('category'),
          $filename
	);

        $this->redirectUnless($request->isXmlHttpRequest(), '@myTransfercar');

        $this->setTemplate('cancelled');
      }
    }
  }
  private function cleanFileName($file_name){ 
    $file_ext = pathinfo($file_name, PATHINFO_EXTENSION); 
    $file_name_str = pathinfo($file_name, PATHINFO_FILENAME); 
     
    // Replaces all spaces with hyphens. 
    $file_name_str = str_replace(' ', '-', $file_name_str); 
    // Removes special chars. 
    $file_name_str = preg_replace('/[^A-Za-z0-9\-\_]/', '', $file_name_str); 
    // Replaces multiple hyphens with single one. 
    $file_name_str = preg_replace('/-+/', '-', $file_name_str); 
    $file_name_str = strlen($file_name_str) > 200 ? substr($file_name_str, 0, 200) : $file_name_str;
    $clean_file_name = $file_name_str.'.'.$file_ext; 
     
     return $clean_file_name; 
  }
 
  public function executeEmailaccept(sfWebRequest $request)
  {
    $this->redirectFromEmail('request/emailaccept', $request);
  }

  public function executeEmailreject(sfWebRequest $request)
  {
    $this->redirectFromEmail('request/emailreject', $request);
  }

  private function redirectFromEmail($url, $request)
  {
    $host = strpos($request->getHost(), 'www.transfercar') === 0 ? str_replace('www.', 'ro.', $request->getHost()) : $request->getHost();
    $host = 'http'.(sfConfig::get('sf_web_debug') ? '' : 's').'://'.$request->getHost();
    $old = sfConfig::get('sf_no_script_name');
    $script_name = sfConfig::get('sf_web_debug') ? 'owner_dev.php' : 'owner.php';
    sfConfig::set('sf_no_script_name', true);
    $path = $this->getController()
        ->genUrl($url.'?key='
                    . $request->getParameter('key')
                    . '&id='.$request->getParameter('id')
                    . '&uid='.$request->getParameter('uid')
                );

    sfConfig::set('sf_no_script_name', $old);

    $this->redirect($host.'/'.$script_name.$path);
  }

  public function executeExplainCancellationFee()
  {
    //
  }

  private function setMetaForListing($listing)
  {
    $metas       = $this->getResponse()->getMetas();
    $description = $this->replacePlaceholders($listing, $metas['description']);
    $title       = $this->replacePlaceholders($listing, $this->getResponse()->getTitle());
    $ogTitle       = $this->replacePlaceholders($listing, $metas['og:title']);
    $ogDescription = $this->replacePlaceholders($listing, $metas['og:description']);
    $ogImage       = $this->replacePlaceholders($listing, $metas['og:image']);
    $ogSitename    = $this->replacePlaceholders($listing, $metas['og:site_name']);

    $this->getResponse()->addMeta('title', $title);
    $this->getResponse()->addMeta('description', $description);
    $this->getResponse()->addMeta('og:title', $ogTitle);
    $this->getResponse()->addMeta('og:description', $ogDescription);
    $this->getResponse()->addMeta('og:site_name', $ogSitename);
    $this->getResponse()->addMeta('og:image', $listing->VehicleType->icon_path(true));
  }

  private function replacePlaceholders($listing, $meta)
  {
    return strtr($meta, array(
      '#DROPOFF_LOCATION#' => $listing->DropoffLocation->name,
      '#PICKUP_LOCATION#' => $listing->PickupLocation->name,
      '#VEHICLE_TYPE#' => $listing->VehicleType->description,
      '#LISTING_ID#' => $listing->id
    ));
  }

}
