<?php

class CancelRequestForm extends RequestNoShowForm
{
    public function configure()
    {
        parent::configure();

        unset($this['internal_comment']);

        if ($this->getOption('ask_reason')) {
          $reasons = Doctrine::getTable('CancellationReason')
            ->createQuery()
            ->where('parent_id is null')
            ->addWhere('only_ro != ?', true)
            ->orderBy('position')
            ->execute();

          $choices = [];
          $category = [];
          $subcategory = [];
          $settings = [];
          foreach ($reasons as $reason) {
            if ($reason->get('Children')->count()) {
              $children = [];
              foreach ($reason->get('Children') as $child) {
                $children[$child->getId()] = $child->get('name');
                $settings[$child->getId()] = [
                  'message' => $child->get('cancellation_fee_message'),
                  'ask_proof' => $child->get('proof_required')
                ];
              }
              $choices[$reason->get('name')] = $children;
            } else {
              $choices[$reason->getId()] = $reason->get('name');
              $settings[$reason->getId()] = [
                'message' => $reason->get('cancellation_fee_message'),
                'ask_proof' => $reason->get('proof_required')
              ];
            }
          }

          $this->widgetSchema['category'] = new sfWidgetFormChoice([
              'choices' => $categories = array_replace(...array_map(function($k, $v){
                return is_array($v) ? [$k => $k] : [$k => $v];
              }, array_keys($choices), array_values($choices))), 
              'expanded' => true, 
              'renderer_options' => ['formatter' => [$this, 'formatCategories']]
          ]);

          $this->widgetSchema['subcategory'] = new sfWidgetFormChoice(['choices' => ['Indicate the type of cancellation'], 'expanded' => false]);

          $this->widgetSchema['file'] = new sfWidgetFormInputFile();
          $this->validatorSchema['file'] = new sfValidatorFile(['required' => false, 'mime_types' => [
            'image/jpeg',
            'image/pjpeg',
            'image/png',
            'image/x-png',
            'image/gif',
            'image/tiff',
            'image/webp',
            'application/pdf',
            'image/jpeg; charset=binary',
            'image/pjpeg; charset=binary',
            'image/png; charset=binary',
            'image/x-png; charset=binary',
            'image/gif; charset=binary',
            'image/tiff; charset=binary',
            'image/webp; charset=binary',
            'application/pdf; charset=binary',
          ]]);

          $this->validatorSchema['category'] = new sfValidatorString();
          $this->validatorSchema['subcategory'] = new sfValidatorString();

          $this->setOption('categories', $choices);
          $this->setOption('settings', $settings);
        }

        $this->validatorSchema['comment'] = new sfValidatorString(['required' => true, 'min_length' => 15]);

        $this->widgetSchema->setLabels(array(
            'comment' => 'Comment',
        ));
    }

    public function bind(array $taintedValues = null, array $taintedFiles = null)
    {
      if (isset($taintedValues['category'])) {
        $categories = $this->getOption('categories');

        if (!is_array($categories[$taintedValues['category']])) {
          $this->validatorSchema['subcategory']->setOption('required', false);
        }
      }
      parent::bind($taintedValues, $taintedFiles);
    }

    public function formatCategories($widget, $inputs)
    {
      $rows = array();
      foreach ($inputs as $input)
      {
        $rows[] = $widget->renderContentTag('div', ' '.str_replace('</label>', $input['input'].'</label>', str_replace('<label ', '<label class="tc-btn-outline-info w-100" ', $input['label'])), ['class' => '']);
      }

      return $widget->renderContentTag('div', implode($widget->getOption('separator'), $rows), array('class' => 'text-center'));
    }
}


class tcWidgetFormSelectRadioExpanded extends sfWidgetFormSelect 
{
  public function render($name, $value = null, $attributes = array(), $errors = array())
  {
    if ('[]' != substr($name, -2))
    {
      $name .= '[]';
    }

    $choices = $this->getOption('choices');
    if ($choices instanceof sfCallable)
    {
      $choices = $choices->call();
    }

    $inputs = $this->doRender($name, $value, $choices, $attributes);

    return $this->formatter($this, $inputs);
  }

  public function formatter($widget, $inputs)
  {
    $rows = array();
    foreach ($inputs as $input)
    {
      $rows[] = $this->renderContentTag('li', 
          $input['input'].
          $this->getOption('label_separator').
          $input['label'],
      array('class' => 'list-group-item'));
    }

    return $this->renderContentTag('ul', implode($this->getOption('separator'), $rows), array('class' => 'list-group'));
  }


  private function doRender($name, $value, $choices, $attributes)
  {
    $inputs = array();
    foreach ($choices as $key => $option)
    {
      if (is_array($option)) 
      {
        $inputs[] = [
          'input' => $this->renderContentTag('label', $key, array('for' => '', 'style' => 'font-weight:bold')).' '.$this->formatter($this, $this->doRender($name, $value, $option, $attributes)),
          'label' => '',
        ];
      }
      else
      {
        $baseAttributes = array(
          'name'  => substr($name, 0, -2),
          'type'  => 'radio',
          'value' => self::escapeOnce($key),
          'id'    => $id = $this->generateId($name, self::escapeOnce($key)),
          'class' => 'me-1'
        );

        if (strval($key) == strval($value === false ? 0 : $value))
        {
          $baseAttributes['checked'] = 'checked';
        }

        $inputs[] = array(
          'input' => '',
          'label' => $this->renderContentTag('label', 
              $this->renderTag('input', array_merge($baseAttributes, $attributes))
              . ' ' . $option. ' <i class="fui-question-circle" data-toggle="tooltip" data-tooltip-style="light" title="'.$option^ls.'"></i>', 
              array('for' => $id)
          ),
        );
      }
    }
    return $inputs;
  }
}
