<?php

class RequestCancelledByDriverForm extends RequestNoShowForm
{
    private static $choices = array(
        'more_48hours' => 'More than 48 hours before pickup time',
        'less_48hours' => 'Less than 48 hours before pickup time',
        'after_pickup' => 'After pickup time (same as no-show)'
    );

    public function configure()
    {
        parent::configure();
        unset($this['comment']);

        $reasons = Doctrine::getTable('CancellationReason')
            ->createQuery()
            ->where('parent_id is null')
            ->addWhere('only_ro != ?', true)
            ->orderBy('position')
            ->execute();

          $choices = [];
          $category = [];
          $subcategory = [];
          $settings = [];
          foreach ($reasons as $reason) {
            if ($reason->get('Children')->count()) {
              $children = [];
              foreach ($reason->get('Children') as $child) {
                $children[$child->getId()] = $child->get('name');
                $settings[$child->getId()] = [
                  'message' => $child->get('cancellation_fee_message'),
                  'ask_proof' => $child->get('proof_required')
                ];
              }
              $choices[$reason->get('name')] = $children;
            } else {
              $choices[$reason->getId()] = $reason->get('name');
              $settings[$reason->getId()] = [
                'message' => $reason->get('cancellation_fee_message'),
                'ask_proof' => $reason->get('proof_required')
              ];
            }
          }

        $this->widgetSchema['category'] = new sfWidgetFormChoice([
              'choices' => $categories = array_replace(...array_map(function($k, $v){
                return is_array($v) ? [$k => $k] : [$k => $v];
              }, array_keys($choices), array_values($choices))), 
              'expanded' => true, 
              'renderer_options' => ['formatter' => [$this, 'formatCategories']]
          ]);

        $this->validatorSchema['category'] = new sfValidatorString();
        $this->validatorSchema['subcategory'] = new sfValidatorString();

        $this->setOption('categories', $choices);
        $this->setOption('settings', $settings);


        $this->widgetSchema['subcategory'] = new sfWidgetFormChoice(['choices' => ['Indicate the type of cancellation'], 'expanded' => false]);


        $this->widgetSchema['canceltime'] = new sfWidgetFormChoice(array(
            'choices' => self::$choices,
            'expanded' => true
        ));

        $this->setDefault('canceltime', 'more_48hours');

        $this->validatorSchema['canceltime'] = new sfValidatorChoice(array(
            'choices' => array_keys(self::$choices)
        ));
    }


    public function bind(array $taintedValues = null, array $taintedFiles = null)
    {
      if (isset($taintedValues['category'])) {
        $categories = $this->getOption('categories');

        if (!is_array($categories[$taintedValues['category']])) {
          $this->validatorSchema['subcategory']->setOption('required', false);
        }
      }

      parent::bind($taintedValues, $taintedFiles);
    }


    public function formatCategories($widget, $inputs)
    {
      $rows = array();
      foreach ($inputs as $input)
      {
        $rows[] = $widget->renderContentTag('div', ' '.str_replace('</label>', $input['input'].'</label>', str_replace('<label ', '<label class="tc-btn-outline-info w-100" ', $input['label'])), ['class' => '']);
      }

      return $widget->renderContentTag('div', implode($widget->getOption('separator'), $rows), array('class' => 'text-center'));
    }
}