<?php

class RequestCancelledByOwnerForm extends RequestNoShowForm
{
    public function configure()
    {
        parent::configure();
        unset($this['comment']);
        $reasons = Doctrine::getTable('CancellationReason')
            ->createQuery()
            ->where('parent_id is not null')
            ->addWhere('only_ro = ?', true)
            ->orderBy('position')
            ->execute();

        $choices = [];
        $settings = [];
        foreach ($reasons as $reason) {
        if ($reason->get('Children')->count()) {
            $children = [];
            foreach ($reason->get('Children') as $child) {
                $children[$child->getId()] = $child->get('name');
                $settings[$child->getId()] = ['message' => $child->get('cancellation_fee_message')];
            }
                $choices[$reason->get('name')] = $children;
            } else {
                $choices[$reason->getId()] = $reason->get('name');
                $settings[$reason->getId()] = ['message' => $reason->get('cancellation_fee_message')];
            }
        }

        $this->widgetSchema['reason'] = new sfWidgetFormChoice([
              'choices' => $categories = array_replace(...array_map(function($k, $v){
                return is_array($v) ? [$k => $k] : [$k => $v];
              }, array_keys($choices), array_values($choices))), 
              'expanded' => true, 
              'renderer_options' => ['formatter' => [$this, 'formatCategories']]
          ]);
        $this->validatorSchema['reason'] = new sfValidatorString();
        $this->widgetSchema->setLabel('reason', 'Please state the reason for cancelling this booking');
        $this->widgetSchema->setLabel('internal_comment', 'Comments');
        $this->widgetSchema->setHelp('internal_comment', '15 characters min.');
    }

    public function formatCategories($widget, $inputs)
    {
      $rows = array();
      foreach ($inputs as $input)
      {
        $rows[] = $widget->renderContentTag('div', ' '.str_replace('</label>', $input['input'].'</label>', str_replace('<label ', '<label class="tc-btn-outline-info w-100" ', $input['label'])), ['class' => '']);
      }

      return $widget->renderContentTag('div', implode($widget->getOption('separator'), $rows), array('class' => 'text-center'));
    }
}
