<?php

use Transfercar\Mailchimp\MailchimpApi;

/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class User extends BaseUser
{
	const STATUS_INACTIVE  = 0; // new, waiting for requests
    const STATUS_ACTIVE    = 1; // accepted, waiting for close
    const STATUS_BLOCKED   = 2; // closed STATUS_ARCHIVED
    const STATUS_SUBSCRIBE = 3; // expired (passed date_return_by)
    const STATUS_DELETED   = 4; // User deleted and anonymized

    const USER_ONLY_DRIVER = 0; // type user - only driver
    const USER_OWNER       = 1; // type user - only owner

    private $defaultContact;

    public function canCreateAirportRelocs()
    {
        return $this->get('can_create_airport_relocs');
    }

    public function isSuspended()
    {
        return $this->get('user_status') == 'suspended';
    }

    /**
     * Get full user name
     */
    public function full_name()
    {
        return $this->firstname.' '.$this->lastname;
    }

    public function getShortName()
    {
        return $this->get('firstname').' '.substr($this->get('lastname'), 0, 1).'.';
    }

    public function getUsername()
    {
        if ($username = parent::rawGet('username')) {
            return $username;
        }

        return $this->getShortName();
    }

    public function getSlug()
    {
        $companySlug = Doctrine_Inflector::urlize($this->get('company'));
        return $companySlug.'-'.$this->get('id');
    }

    /**
     * alias to use User type the same way as OwnerUser
     */
    public function getOwnerId()
    {
        return $this->getId();
    }

    /**
     * Alias of full_name() for the admin gen
     */
    public function getFullName()
    {
        return $this->full_name();
    }

    /**
     * Use method above to the same action
     */
    public function __toString()
    {
        return $this->full_name();
    }

    public function getDefaultContact()
    {
        if (null === $this->defaultContact) {
            $this->defaultContact = Doctrine::getTable('OwnerContact')->createQuery()
                ->where('owner_id = ?', $this->getId())
                ->addWhere('is_default = 1')
                ->fetchOne();

            if (!$this->defaultContact) {
                $this->defaultContact = Doctrine::getTable('OwnerContact')->createQuery()
                    ->where('owner_id = ?', $this->getId())
                    ->orderBy('id asc')
                    ->fetchOne();
            }
        }

        return $this->defaultContact;
    }

    /**
     * Get detailed info about user (used in emails)
     * @return string User detailed info
     */
    public function detailedInfo()
    {
        $result = array();

        if($this->company){
            $result[] = 'company: '.$this->company;
        } else {
            $result[] = $this->full_name();
        }

        $result[] = "email: {$this->email}";

        if($this->contact_phone){
            $result[] = 'phone: '.$this->contact_phone;
        }

        if($this->mobile_phone){
            $result[] = 'cell: '.$this->mobile_phone;
        }

        return implode(', ', $result);
    }

    public function detailedInfoArray()
    {
        $result['username'] = $this->username;
        $result['email'] = $this->email;
        $result['company'] = $this->company;
        $result['full_name'] = $this->full_name();
        $result['contact_phone'] = $this->contact_phone;
        $result['mobile_phone'] = $this->mobile_phone;

        return $result;
    }
    /**
     * Calculate user rank (in range from 0 to 5) based
     * on average_rating field
     *
     * We make following ranges (with 0.20 step):
     * 0.00 - 0.10 - 0 stars
     * 0.10 - 0.30 - 1 star
     * 0.30 - 0.50 - 2 stars
     * 0.50 - 0.70 - 3 stars
     * 0.70 - 0.90 - 4 stars
     * 0.90 - 1.00 - 5 stars
     *
     * @return int
     */
    public function star_count()
    {
        $rank = intval($this->average_rating * 100); // 0..100
        $rank += 10; // 10..110
        $rank = intval($rank / 20); // 0..5

        return $rank;
    }

    /**
     * Calculate number of user relocations (rated requests)
     *
     * @return int
     */
    public function reloc_count()
    {
        $result = 0;

        foreach($this->Requests as $request){
            if($request->status == Request::STATUS_RATED){
                $result++;
            }
        }

        return $result;
    }

    public function getNbCancellations()
    {
        return Doctrine::getTable('Request')
            ->createQuery('r')
            ->where('driver_id = ?', $this->id)
            ->whereIn('status', [Request::STATUS_DRIVER_CANCELED, Request::STATUS_CANCELED_BY_DRIVER])
            ->addWhere('(is_withdrawn is null or is_withdrawn = ?)', false)
            ->count();
    }

    public function getNbDisruptedBookings()
    {
        return Doctrine::getTable('Request')
            ->createQuery('r')
            ->where('driver_id = ?', $this->id)
            ->whereIn('status', [Request::STATUS_DRIVER_CANCELED, Request::STATUS_CANCELED_BY_DRIVER])
            ->addWhere('is_withdrawn = ?', true)
            ->count();
    }


    /**
    *Function used in backend/users/edit
    *@return array of rated requests
    */


    public function getRatings()
    {
        // Get all closed requests given user
        $requests = Doctrine_Query::create()
            ->from('Request')
            ->where('driver_id = ?', $this->id)
            ->addWhere('status = ?', Request::STATUS_RATED)
            ->execute();


        return $requests;
    }

    /**
    * Update information about average rating
    */

    public function recalculate_avg_rating()
    {
        // Get all closed requests given user
        $baseQuery = Doctrine_Query::create()
            ->from('Request')
            ->where('driver_id = ?', $this->id)
            ->addWhere('status = ?', Request::STATUS_RATED)
            ->addWhere('(rating is null or rating != 0)') // neutral ratings are ignored
        ;

        $query = clone $baseQuery;
        $result = $query
            ->select('SUM(rating) as total_rank')
            ->execute(array(), Doctrine::HYDRATE_NONE);

        $total_rank = $result[0][0];

        $query = clone $baseQuery;
        $result = $query
            ->select('COUNT(*) as nb_requests')
            ->execute(array(), Doctrine::HYDRATE_NONE);

        $nb_requests = $result[0][0];

        // if a driver has only neutral ratings, we do not update the rating score.
        if ($nb_requests) {
            $this->average_rating = $total_rank / ($nb_requests);
        }
        $this->save();
    }

    public function recalculateRORating()
    {
        $rating = Doctrine::getTable('RelocationReview')->getRoRating($this);
        $this->set('average_rating', $rating);
        $this->save();
    }

    public function getNbReviews()
    {
        return Doctrine::getTable('RelocationReview')->getNbReviews($this);
    }


    /**
     * Calculates user age
     *
     * @returns int User age
     */
    public function getAge()
    {
          $birthday = strtotime($this->date_of_birth);
          return intval((time() - $birthday) / (86400 * 365.25));
    }


    /**
     * Set default values on user registration
     */
    public function preInsert($event)
    {
        $this->account_balance = 0.0;
	    $this->user_status     = 'inactive';
	    $this->when_registered = date('Y-m-d H:i:s');
    }

    /**
     * Check if users details has been blocked before
     */
    public function preSave($event)
    {
        $check = $event->getInvoker()->exists();
        foreach (['email', 'firstname', 'lastname', 'date_of_birth', 'mobile_phone'] as $key) {
            
            if (in_array($key, array_keys($this->modifiedFields())))
            {
                $check = true;
                break;
            }
        }
	
        if ($check && $this->getTable()->isSuspect($event->getInvoker())) {
            $event->getInvoker()->set('internal_status', 1);
           
            return;
        }        
    }

    /**
     *
     */
    public function getLastRelocation(){
        $q = Doctrine_Query::create()->select('l.created_at as latest_date')
                ->from('Listing l')
                ->where('l.owner_id = ? and accepted_request_id > 0', $this->get('id'))
                ->orderBy('latest_date DESC')
                ->limit(1);
        $result = $q->fetchOne(array(), Doctrine::HYDRATE_ARRAY);
        return $result['latest_date'];
    }

    public function flagAsDeleted()
    {
        $conn = $this->getTable()->getConnection();

        try
        {
            $conn->beginTransaction();

            $this->set('user_status', 'deleted');
            $this->unsubscribeNewsletter('nz');
            $this->unsubscribeNewsletter('au');
            $this->unsubscribeNewsletter('us');
            $this->anonymize();

            foreach ($this->get('Requests') as $request)
            {
                $request->set('status', Request::STATUS_DELETED);
            }

            foreach ($this->get('Listings') as $listing)
            {
                $listing->set('status', Listing::STATUS_DELETED);
            }

            $this->get('NotifyRoutes')->delete($conn);
            $this->get('CreditCards')->delete($conn);
            $this->get('Terms')->delete($conn);
            // $this->get('RelocCount')->delete($conn);

            $this->get('Requests')->save($conn);
            $this->get('Listings')->save($conn);
            $this->save($conn);

            $conn->commit();
        }
        catch (Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    protected function unsubscribeNewsletter($domain)
    {
        //remove user from Mailchimp Web service
        $mc = new MailchimpApi();
        $mc->unsubscribe(sfConfig::get('app_mailchimp_'.$domain.'_list_apikey'), $this->get('email'));
    }

    /**
     * Overrides the default behavior to encrypt the password using bcrypt
     */
    public function setPassword($password)
    {
        $salt = sprintf('$2y$%02d$', 12);
        $salt .= hash('whirlpool', uniqid(mt_rand(), true));

        parent::rawSet('password', crypt($password, $salt));
    }

    public function checkPassword($password)
    {
        return crypt($password, $this->get('password')) == $this->get('password');
    }

    public function generateRandomKey($algorithm = 'whirlpool')
    {
        return hash($algorithm, uniqid(mt_rand(), true));
    }

    protected function anonymize()
    {
        $this->set('email', 'anonymous@example.com');
        $this->set('new_email', null);
        $this->set('new_email_confirm', null);
        $this->set('username', 'anonymous@'.uniqid());
        $this->set('firstname', 'anonymous');
        $this->set('lastname', 'user');
        $this->set('date_of_birth', date('Y-m-d', 0));
        $this->set('company', null);
        $this->set('contact_phone', null);
        $this->set('mobile_phone', null);
        $this->set('address_line1', '-');
        $this->set('address_line2', '');
        $this->set('suburb', '-');
        $this->set('city', '-');
        $this->set('postcode', '0000');
        $this->set('password', '__no_passwd__');
        $this->set('account_balance', 0);
        $this->set('email_newsletter', false);
        $this->set('email_country_newsletter', false);
        $this->set('email_notify_new_reloc', 'none');
        $this->set('email_country_new_reloc', 'none');
        $this->set('sms_number', null);
        $this->set('sms_number_confirmed', false);
        $this->set('sms_notify_update_req', false);
        $this->set('sms_notify_new_reloc', 'none');
    }

    public function isEmailBounced()
    {
        return $this->get('is_hardbounced') || $this->get('emails_softbounced') > 10;
    }

    public function mustCreditAccount($limit = 0)
    {
        $this->refresh();
        return $this->get('payment_cc') && $this->get('account_balance') < $limit;
    }

    public function isVerified()
    {
        return $this->get('user_status') === 'active' && !$this->get('new_email');
    }

    public function isAccountComplete()
    {
        return $this->get('user_status') === 'active'
            && $this->get('date_of_birth')
            && $this->get('license')
            && $this->get('mobile_phone');
    }

    public function hasPhoneVerified()
    {
        return $this->get('sms_number_confirmed');
    }

    public function isFullyVerified()
    {
        return $this->isVerified() && $this->hasPhoneVerified();
    }

    public function getRefundRate()
    {
        $query = Doctrine_Query::create()
            ->from('Request r')
            ->innerJoin('r.Listing l')
            ->where('l.owner_id = ?', $this->getId())
            ->addWhere('r.when_accepted is not null')
            ->addWhere('r.when_accepted > ?', date('Y-m-d', strtotime('-1 year')));

        $totalListings = $query->count();
        $totalRefunded = $query
            ->addWhere('r.internal_status & ?', Request::INTERNAL_STATUS_REFUNDED)
            ->count();

        return $totalRefunded / $totalListings;
    }

    public function confirmEmail()
    {
        $this->user_status = 'active';
        $this->last_login = date('Y-m-d H:i:s', time());
        $this->activation_code = mt_rand(100000, 999999); // used by "remember me" option

        if ($this->new_email)
        {
            $this->confirmNewEmail();
        }
    }

    public function confirmNewEmail()
    {
        $this->email = $this->new_email;
        $this->new_email = null;
        $this->new_email_confirm = null;
        $this->is_hardbounced = 0;
        $this->emails_softbounced = 0;
    }

    public function confirmSmsNumber($code)
    {
        if (trim($code) == $this->get('sms_number'))
        {
            $this->set('sms_number', null);
            $this->set('sms_number_confirmed', true);
            $this->save();

            return true;
        }

        return false;
    }

    public function getOpenedRequests()
    {
        return Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->innerJoin('l.PickupLocation p')
            ->innerJoin('l.DropoffLocation d')
            ->innerJoin('l.VehicleType')
            ->innerJoin('l.Owner o')
            ->leftJoin('l.AcceptedRequest ar')
            ->leftJoin('r.Review rr')
            ->where('r.driver_id = ?', $this->getId())
            ->addWhere('datediff(now(), GREATEST(r.created_at, r.pickup_date, r.drop_off_date, COALESCE(r.when_accepted, 0), COALESCE(r.cancelled_at, 0))) <= 60')
            ->orderBy('r.created_at DESC')
            ->execute();
    }

    public function getPendingRequests()
    {
        return Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->innerJoin('l.VehicleType')
            ->where('r.driver_id = ?', $this->get('id'))
            ->addWhere('r.status = ?', Request::STATUS_PENDING)
            ->execute();
    }

    public function getClosedRequests()
    {
        return Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->innerJoin('l.PickupLocation p')
            ->innerJoin('l.DropoffLocation d')
            ->innerJoin('l.VehicleType')
            ->innerJoin('l.Owner o')
            ->leftJoin('l.AcceptedRequest ar')
            ->leftJoin('r.Review rr')
            ->where('r.driver_id = ?', $this->getId())
            ->addWhere('datediff(now(), GREATEST(r.created_at, r.pickup_date, r.drop_off_date, COALESCE(r.when_accepted, 0), COALESCE(r.cancelled_at, 0))) > 60')
            ->orderBy('r.created_at DESC')
            ->execute();
    }

    public function getPhone()
    {
        $phones = array();

        if ($this->get('contact_phone')) {
            $phones[] = 'Phone: '.$this->get('contact_phone');
        }
        if ($this->get('mobile_phone')) {
            $phones[] = 'Mobile: '.$this->get('mobile_phone');
        }

        return explode(",\n", $phones);
    }

    public function getDefaultTimeZone()
    {
        $country = $this->get('Country');

        switch ($country->get('domain'))
        {
            case 'au':
                return 'Australia/Sydney';

            case 'us':
                return 'America/Adak';

            case 'nz':
                return 'Pacific/Auckland';

            default:
                return 'UTC';
        }
    }

    public function getEmailToParams()
    {
        return array(
            'email'  => $this->get('email'),
            'name'   => $this->get('firstname'),
            'format' => 'html',
            'id'     => $this->get('id'),
            'lang'   => $this->get('culture') == 'fr_CA' ? 'fr' : 'en'
        );
    }

    public function getReviews()
    {
        if (!$this->get('is_owner')) {
            return array();
        }

        return Doctrine::getTable('RelocationReview')
            ->findForROQuery($this)
            ->orderBy('r.created_at desc')
            ->limit(10)
            ->execute();
    }

    public function getAverageRatingLabel()
    {
        $labels = array(
            1 => "Awful",
            2 => "Poor",
            3 => "Average",
            4 => "Good",
            5 => "Excellent"
        );

        $rating = max(1, ceil($this->get('average_rating')));

        return $labels[$rating];
    }

    public function reindexListings()
    {
        $listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->where('l.owner_id = ?', $this->getId())
            ->addWhere('l.status = ?', Listing::STATUS_NEW)
            ->execute();

        foreach ($listings as $listing) {
            tcElasticaListing::listenToListingUpdated(new sfEvent($listing, ''));
        }
    }

    public function getLastCreditCard()
    {
        return Doctrine_Query::create()
            ->from('CreditCard')
            ->where('owner_id = ?', $this->getId())
            ->addWhere('holder_name like ?', 'src_%')
            ->orderBy('id DESC')
            ->limit(1)
            ->fetchOne();
    }

    public function isBannedFor($ro)
    {
        $nb = Doctrine::getTable('RoBan')
            ->createQuery('b')
            ->where('b.driver_id = ?', $this->getId())
            ->addWhere('b.ro_id=?', $ro)
            ->count();

        return $nb > 0;
    }
}
